/****************************************************************************/
/*                                                                          */
/*  VolVis is a volume visualization system for investigating, manipulating */
/*  and rendering geometric and volumetric data.                            */
/*                                                                          */
/*  Copyright (C) 1993 by the Research Foundation of the State University   */
/*                            of New York                                   */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 1, or (at your option)     */
/*  any later version.                                                      */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               */
/*                                                                          */
/*  For information on VolVis, contact us at:                               */
/*                                                                          */
/*                volvis@cs.sunysb.edu                         (email)      */
/*                                                                          */
/*                Lisa Sobierajski & Ricardo Avila             (US Mail)    */
/*                Department of Computer Science                            */
/*                State University of New York at Stony Brook               */
/*                Stony Brook, New York  11794-4400                         */
/*                                                                          */
/****************************************************************************/




/*
 *                  File: C_dataglove.h
 *                Author: Theodossiou, Andreas
 *                  Date: 05/11/1993
 *           Description: DataGlove U.I.
 *  Modification History:
 *
 *         who?               when?           why?
 *    -----------------------------------------------------------
 *
 */
char *dataglove_commands[C_MAX_COMMANDS] = {"Repeat 60",
	 				    "Repeat 30",
                                            "One Shot", 
                                            "Set Angle Output",
                                            "Set Flex Output", 
                                            "UserRead",        
                                            "Write To User Port",
                                            "Load Table",        
                                            "Set LED Level",     
                                            "Read LED Level",    
                                            "SystemID",          
                                            "ColdReset",         
                                            "WarmReset",         
                                            "UserIRQ",           
                                            "Send to Polhemus",  
                                            "Set Polhemus units",
                                            "Number of sensors", 
                                            "Load Calibration",  
                                            "Load Gestures",
                                            "QUIT"};

char* dataglove_baud_rates[C_MAX_BAUD_RATE] = {"19200",
                                               "9600",
                                               "4800",
                                               "2400",
                                               "1200",
                                               "300"};

int dataglove_baud_rates_int[C_MAX_BAUD_RATE] = {19200,
                                                 9600,
                                                 4800,
                                                 2400,
                                                 1200,
                                                 300};

