/****************************************************************************/
/*                                                                          */
/*  VolVis is a volume visualization system for investigating, manipulating */
/*  and rendering geometric and volumetric data.                            */
/*                                                                          */
/*  Copyright (C) 1993 by the Research Foundation of the State University   */
/*                            of New York                                   */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 1, or (at your option)     */
/*  any later version.                                                      */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               */
/*                                                                          */
/*  For information on VolVis, contact us at:                               */
/*                                                                          */
/*                volvis@cs.sunysb.edu                         (email)      */
/*                                                                          */
/*                Lisa Sobierajski & Ricardo Avila             (US Mail)    */
/*                Department of Computer Science                            */
/*                State University of New York at Stony Brook               */
/*                Stony Brook, New York  11794-4400                         */
/*                                                                          */
/****************************************************************************/




/*
 *                  File: C_spaceball.h
 *                Author: Theodossiou, Andreas
 *                  Date: 05/11/1993
 *           Description: Spaceball U.I.
 *  Modification History:
 *
 *         who?               when?           why?
 *    -----------------------------------------------------------
 *
 */
char *spaceball_label_names[C_SPACEBALL_MAX_LABELS] = {"View           Mode",
                                                     "Trans         Mode",
                                                     "Rot             Mode",
                                                     "VCR           Mode",
                                                     "VCR Loop Mode",
                                                     "VCR Tape Name",
                                                     "VCR Tape  Dir"};
char *spaceball_view_names[C_MAX_VIEWM]        = {"Object     ",
                                                "Eyepoint",
                                                "Orbit",
                                                "Orbit   Direction",
                                                "Pullthru"};
char *spaceball_trans_names[C_MAX_TRANSM]      = {"Trans Off",
                                                "Trans Dominant",
                                                "Trans On",
                                                "Trans Z Only",
                                                 "Super Dominant"};
char *spaceball_rot_names[C_MAX_ROTM]          = {"Rot Off",
                                                "Rot Dominant",
                                                "Rot On",
                                                "Super Dominant"};
char *spaceball_vcr_names[C_MAX_VCRM]          = {"Recording          ",
                                                "Stopped",
                                                "Playing"};
char *spaceball_vcr_loop_names[C_MAX_VCRLOOPM] = {"One Play            ",
                                                "Repeat",
                                                "Bounce"};
                                    
